# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('opal', '0024_auto_20160826_1906'),
        ('opal', '0024_auto_20160826_1429'),
    ]

    operations = [
    ]
