      <div class="modal-header">
	    <button type="button" class="close" ng-click="cancel()">×</button>
	    <h3>Delete item?</h3>
      </div>
      <div class="modal-body"><p>Do you want to delete this item?</p></div>
      <div class="modal-footer">
	    <button class="btn btn-primary" ng-click="destroy()">Yes</button>
	    <button class="btn" ng-click="cancel()">No</button>
      </div>
