{% extends 'base.html' %}
{% block content %}
  <h1 class="content-offset">
    Sorry, we couldn't find that page.
  </h1>
  <p>
    Perhaps you can find what you're looking for in the menu above?
  </p>
{% endblock %}
