# cryptography is required for heroku deployment
cryptography==1.3.2
Django==1.8.13
coverage==3.6
dj-database-url==0.2.1
gunicorn==0.17.4
psycopg2==2.5
dj-static==0.0.6
django-reversion==1.8.7
django-axes==1.4.0
ffs==0.0.8.1
letter==0.4.1
requests==2.7.0
djangorestframework==3.2.2
django-compressor==1.5
python-dateutil==2.4.2
django-celery==3.1.17
celery==3.1.19
opal=={{version}}
